using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Camera
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {

        BasicEffect be;
        Matrix view;
        
        Matrix projection;
        Model asteroid;
        Model ship;
        Matrix shipAttitude;
        Camera cam;

        GraphicsDeviceManager graphics;

        Asteroid[] block;

        int numblocks;

        RenderTarget2D rt;
        SpriteBatch sb;
        Effect post;
        EffectParameter postTime;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            
        }

        void InitializeEffect()
        {
            be = new BasicEffect(graphics.GraphicsDevice, null);
            be.LightingEnabled = false;
            be.VertexColorEnabled = true;


        }

        void InitializeField()
        {
            numblocks = 400;
            Vector3 blockpos; 
            Quaternion blockor;
            Vector3 blockL;
            Vector3 blockv;

            Random r = new Random();
            block = new Asteroid[numblocks];

            for (int i = 0; i < numblocks; i++)
            {
                blockv = new Vector3(0, 0, 0);
                blockpos.X = 500 - (float)r.NextDouble() * 1000;
                blockpos.Y = 500 - (float)r.NextDouble() * 1000;
                blockpos.Z = 500 - (float)r.NextDouble() * 1000;
                blockor.W = (float)(r.NextDouble() * Math.PI);
                blockor.X = (float)(r.NextDouble() * Math.PI);
                blockor.Y = (float)(r.NextDouble() * Math.PI);
                blockor.Z = (float)(r.NextDouble() * Math.PI);
                blockor.Normalize();
            
                blockL.X = (float)(r.NextDouble() * Math.PI);
                blockL.Y = (float)(r.NextDouble() * Math.PI);
                blockL.Z = (float)(r.NextDouble() * Math.PI);
//                blockL[i].Normalize();

                block[i] = new Asteroid(blockpos, blockv, blockor, blockL);
            }
        }
        public void InitializePostproc()
        {
            sb = new SpriteBatch(GraphicsDevice);

            rt = new RenderTarget2D(GraphicsDevice,
                GraphicsDevice.Viewport.Width,
                GraphicsDevice.Viewport.Height, 1,
                GraphicsDevice.DisplayMode.Format);
            post = Content.Load<Effect>("Content\\image");
            postTime = post.Parameters["time"];
            postTime.SetValue(0.1f);
        }
        public void InitializeCamera()
        {
            Vector3 camerapos =
                new Vector3(0.0f, 0.0f, -300.0f);
            Vector3 lookat =
                new Vector3(0.0f, 0.0f, -299.0f);

            Vector3 up = Vector3.Up;

            Matrix.CreatePerspectiveFieldOfView(1.5f*MathHelper.PiOver4/*angle*/, 1.0f/*aspect ratio*/,
                    1.0f /*near*/, 100000.0f /*far*/,
                out projection /*output*/);

            Matrix.CreateLookAt(ref camerapos, ref lookat, ref up, out view);

            cam = new Camera();
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            InitializePostproc();
            InitializeCamera();
            InitializeEffect();
            InitializeField();
            base.Initialize();

        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadContent()
        {
            

            // TODO: Load any ResourceManagementMode.Manual content
            asteroid = Content.Load<Model>("Content\\asteroid1model");

            // ship model by the artist, whose name I don't know :(
            ship = Content.Load<Model>("Content\\Models\\Ship_06");
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadContent()
        {
           
            // TODO: Unload any ResourceManagementMode.Manual content
        }

        
        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {

            for (int i = 0; i < numblocks; i++)
                block[i].Update(gameTime);

            shipAttitude = Matrix.CreateRotationY(3.14f)*Matrix.CreateRotationX(0.6f);
             
            // Allows the game to exit
            KeyboardState kb = Keyboard.GetState();
            cam.run(); // always move forward
             
            if (kb.IsKeyDown(Keys.Space))
                cam.run();
            if (kb.IsKeyDown(Keys.J))
                cam.strafeL();

            if (kb.IsKeyDown(Keys.K))
                cam.strafeR();

            if (kb.IsKeyDown(Keys.Q))
            {
                cam.turnL();
                shipAttitude = shipAttitude * Matrix.CreateRotationZ(0.2f);
            }
            if (kb.IsKeyDown(Keys.W))
            {
                cam.turnR();
                shipAttitude = shipAttitude * Matrix.CreateRotationZ(-0.2f);
            }
            if (kb.IsKeyDown(Keys.S))
                cam.rollL();
            if (kb.IsKeyDown(Keys.A))
                cam.rollR();
            if (kb.IsKeyDown(Keys.E))
            {
                cam.Climb();
                shipAttitude = shipAttitude * Matrix.CreateRotationX(0.2f);
            }
            if (kb.IsKeyDown(Keys.D))
            {
                cam.Dive();
                shipAttitude = shipAttitude * Matrix.CreateRotationX(-0.2f);
            }
            // TODO: Add your update logic here

            view = cam.getView();
            base.Update(gameTime);
        }
        //----------------------------------------------------------------------------------------------------
        void DrawOne(Model m, Matrix world)
        {

            foreach (ModelMesh mesh in m.Meshes)
            {
                foreach (BasicEffect be in mesh.Effects)
                {
                    be.EnableDefaultLighting();
                    be.Projection = projection;
                    be.View = view;
                    be.World = world *
                        mesh.ParentBone.Transform;

                }
                mesh.Draw();
            }
        }

        
        void DrawField()
        {
          
            for (int i = 0; i < numblocks; i++)
            {
                block[i].Draw(asteroid, projection, view);
                
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            postTime.SetValue((float)gameTime.TotalGameTime.TotalSeconds);
            GraphicsDevice.SetRenderTarget(0, rt);
            graphics.GraphicsDevice.Clear(Color.Black);

            // TODO: Add your drawing code here
            DrawField();
            Matrix shipRot;
            Matrix shipTrans;
            shipRot = Matrix.CreateFromQuaternion(cam.q);
            shipTrans = Matrix.CreateTranslation(cam.pos)*Matrix.CreateTranslation(15*cam.front);
            DrawOne(ship,shipAttitude*shipRot*shipTrans);
            base.Draw(gameTime);
            GraphicsDevice.SetRenderTarget(0, null);
            graphics.GraphicsDevice.Clear(Color.Black);
            post.Begin();
            sb.Begin(SpriteBlendMode.None, SpriteSortMode.Immediate, SaveStateMode.None);
            EffectPass ep = post.CurrentTechnique.Passes[0];
            ep.Begin();
            sb.Draw(rt.GetTexture(), Vector2.Zero, Color.White);
            ep.End();
            sb.End();
            post.End();

        }
    }
}
